/*  timer.cpp
 *  This program demonstrates timer callbacks to control display updates 
 *  using bitmap graphics.  
 *
 *  Modified to use timer in bmptext demo -- MGR 07/09/28
 */

// bmptext.cpp
// by Glenn G. Chappell
// September 2003
//
// For CS 381
// GLUT Bitmap Text Demo

#include <iostream>
using std::cerr;
using std::endl;
#include <string>
using std::string;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <sstream>
using std::ostringstream;
#include <iomanip>
using std::setfill;
using std::setw;
#include <GL/glut.h> // GLUT stuff, includes OpenGL headers as well


#define FPS 1
// Global variables
// Window/viewport
const int startwinsize = 400; // Starting window width & height, in pixels

// Keyboard
const int ESCKEY = 27;        // ASCII value of escape character

// Globals affecting display
bool showtext = true;         // True if text (help) is shown
bool startclock = true;       // True if clock is running
int hour=0, min=0, sec=0;     // time variables
string timestring="00:00:00";

// printbitmap
// Prints the given string at the given raster position
//  using GLUT bitmap fonts.
// You probably don't want any rotations in the model/view
//  transformation when calling this function.
void printbitmap(const string msg, double x, double y)
{
   glRasterPos2d(x, y);
   for (string::const_iterator ii = msg.begin(); ii != msg.end(); ++ii)
   {
      glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *ii);
   }
}

// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT); //clear display
   if (showtext) // Draw text
   {
      printbitmap("GLUT Timer Demo", 0.05, 0.95);
      printbitmap("H     Toggle help", 0.05, 0.85);  // double space 
      printbitmap("S     Start/stop clock", 0.05, 0.8);
      printbitmap("R     Reset clock", 0.05, 0.75);
      printbitmap("Esc   Quit", 0.05, 0.7);
   }
   printbitmap(timestring, 0.4, 0.5); // time string
   glutSwapBuffers();
}

void myTimer(int v)     // timer function to update time
{
    ostringstream timestream;      // used to format time string
    if (startclock) sec++;         // increment second
    if (sec==60) { min++; sec=0; } // increment minute
    if (min==60) { hour=hour++ % 24; min=0; } //increment hour
    timestream << setfill('0') << setw(2) << hour <<":" << setw(2)
               << min << ":" << setw(2) << sec;
    timestring=timestream.str();  //convert stream to string
    glutPostRedisplay();
    glutTimerFunc(1000/v, myTimer, v);  //repost timer 
}

// keyboard
// The GLUT keyboard function
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
   case ESCKEY:  // ESC: Quit
      exit(0);
      break;
   case 'h':  // h - toggle text (help)
   case 'H':  // upper- and lower-case are different!
      showtext = !showtext;
      break;
   case 's':  // s - toggle clock on/off
   case 'S':
      startclock = !startclock;
      break;
   case 'r':  // r - reset clock
   case 'R':
      hour=min=sec=0;
      break;
   }
   glutPostRedisplay();
}


// init
// Initializes GL states
// Called by main after window creation
void init()
{
   // Set background color
   glClearColor(0.9, 0.9, 0.1, 0.0);  // yellow
   // Set drawing/text color
   glColor3d(0.1, 0.1, 0.9);  // blue
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0.0, 1.0, 0.0, 1.0);
   glMatrixMode(GL_MODELVIEW);  // Always go back to model/view mode
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - GLUT Timer Demo");

   // Initialize GL states & register callbacks
   init();
   glutDisplayFunc(display); 
   //glutIdleFunc(idle);
   glutKeyboardFunc(keyboard);
   glutTimerFunc(1000,myTimer,FPS);
   // Do something
   glutMainLoop();

   return 0;
}
